#include "view/main_window.h"
#include "view/swipe_lock.h"
#include "$(appName)-ref.h"
#include "$(appName)-ref_common.h"

static struct
{
	Evas_Object *win;
	Evas_Object *background;
	Evas_Object *swipe_lock;
	swipe_unlock_cb swipe_unlock_cb;
	int screen_width;
	int screen_height;
} s_main_window_data = {
	.win = NULL,
	.background = NULL,
	.swipe_lock = NULL,
	.swipe_unlock_cb = NULL,
	.screen_width = 0,
	.screen_height = 0
};

static void __delete_cb(void *data, Evas_Object *obj, void *event_info);
static bool __create_background(const int width, const int height);
static void __set_background_transparency(double alpha_scale);

static bool __create_swipe(const int width, const int height);
static void __swipe_drag_cb(void *data, Evas_Object *obj, const char *emission, const char *source);
static void __swipe_drag_end_cb(void *data, Evas_Object *obj, const char *emission, const char *source);


bool
create_main_window(void)
{

	s_main_window_data.win = elm_win_add(NULL, PACKAGE, ELM_WIN_DOCK);

	if (!s_main_window_data.win) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failure allocating resource: main window");
		return false;
	}

	elm_win_conformant_set(s_main_window_data.win, EINA_FALSE);
	elm_win_screen_size_get(s_main_window_data.win, NULL, NULL, &s_main_window_data.screen_width, &s_main_window_data.screen_height);
	evas_object_resize(s_main_window_data.win, s_main_window_data.screen_width, s_main_window_data.screen_height);

	/* default alpha is 0: */
	elm_win_alpha_set(s_main_window_data.win, EINA_TRUE);

	evas_object_smart_callback_add(s_main_window_data.win, "delete,request", __delete_cb, NULL);

	if (!__create_background(s_main_window_data.screen_width, s_main_window_data.screen_height) ||
		!__create_swipe(s_main_window_data.screen_width, s_main_window_data.screen_height)
		) {
			return false;
	}


	/* Show window after base gui is set up */
	evas_object_show(s_main_window_data.win);

	return true;
}

void
destroy_main_window(void)
{
	evas_object_del(s_main_window_data.win);
	s_main_window_data.win = NULL;
}

void
add_main_window_callbacks(swipe_unlock_cb unlock_cb)
{
	if (!s_main_window_data.swipe_unlock_cb || unlock_cb) {
		_add_swipe_lock_callbacks(s_main_window_data.swipe_lock, __swipe_drag_cb, __swipe_drag_end_cb);
		s_main_window_data.swipe_unlock_cb = unlock_cb;
	}
}

void
del_main_window_callbacks(void)
{
	_del_swipe_lock_callbacks(s_main_window_data.swipe_lock, __swipe_drag_cb, __swipe_drag_end_cb);
	s_main_window_data.swipe_unlock_cb = NULL;
}

void
activate_main_window_swipe_lock(void)
{
	evas_object_show(s_main_window_data.win);
	evas_object_show(s_main_window_data.swipe_lock);
}

void
deactivate_main_window_swipe_lock(void)
{
	evas_object_hide(s_main_window_data.swipe_lock);
	evas_object_hide(s_main_window_data.win);
}

void
set_main_window_current_wallpaper(const char *file_path)
{
	char default_file[PATH_MAX] = {0,};

	if (!elm_bg_file_set(s_main_window_data.background, file_path, NULL)) {
		compile_resource_path(DEAFULT_BG_IMAGE, default_file);

		dlog_print(DLOG_ERROR, LOG_TAG, "Failure to set background image %s, setting default: %s", file_path, default_file);
		elm_bg_file_set(s_main_window_data.background, default_file, NULL);
	}
}


static void
__delete_cb(void *data, Evas_Object *obj, void *event_info)
{
	ui_app_exit();
}

static bool
__create_background(const int width, const int height)
{
	s_main_window_data.background = elm_bg_add(s_main_window_data.win);
	if (!s_main_window_data.background) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failure allocating resource: main window background");
		return false;
	}

	elm_win_resize_object_add(s_main_window_data.win, s_main_window_data.background);
	elm_bg_load_size_set(s_main_window_data.background, width, height);
	evas_object_show(s_main_window_data.background);

	return true;
}

static void
__set_background_transparency(double alpha_scale)
{
	int scaled_alpha = (int)(alpha_scale*(double)255);
	evas_object_color_set(s_main_window_data.background, 255, 255, 255, scaled_alpha);
}

static bool
__create_swipe(const int width, const int height)
{
	s_main_window_data.swipe_lock = _create_swipe_lock(s_main_window_data.win);
	if (!s_main_window_data.swipe_lock) {
		dlog_print(DLOG_ERROR, LOG_TAG, "Failure allocating resource: swipe_lock");
		return false;
	}

	evas_object_move(s_main_window_data.swipe_lock, 0, 0);
	evas_object_resize(s_main_window_data.swipe_lock, width, height);

	return true;
}

static void
__swipe_drag_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	double dy, h_ratio;

	if (!s_main_window_data.swipe_unlock_cb)
		return;

	_get_swipe_lock_drag_position(s_main_window_data.swipe_lock, NULL, &dy);

	h_ratio = fabs(dy)/(double)s_main_window_data.screen_height;

	__set_background_transparency(1.0 - h_ratio/SWIPE_UNLOCK_PERCENTAGE);

	if (h_ratio >= SWIPE_UNLOCK_PERCENTAGE) {
		s_main_window_data.swipe_unlock_cb();
	}
}

static void
__swipe_drag_end_cb(void *data, Evas_Object *obj, const char *emission, const char *source)
{
	_reset_swipe_lock_drag(s_main_window_data.swipe_lock);
	__set_background_transparency(1.0);
}
